#include <Wire.h>
#include "IRLibAll.h"
#include <LiquidCrystal_I2C.h>

unsigned int IR_PIN = 11;                   // IR receiver input pin

IRrecv myReceiver(IR_PIN);                  // IR receiver object to listen on pin IR_PIN

IRdecode myDecoder;                         // IR decoder object 

LiquidCrystal_I2C lcd(0x3f,2,1,0,4,5,6,7);  // Set the LCD I2C interface

//Volume encoder settings:
const int dBperStep = 2;                    // attenuation per step in dB
const int EncoderResolution = 256 - 1;      // Encoder resolution 4 x <pulses per revolution>
const int MAXAttenuation = 32 - 1;          // Maximum number of attenuation steps
const int EncoderToAttenuationRatio = EncoderResolution/MAXAttenuation;
int VolEncoderCurrState;                    // Encoder state (quadrature code)
int VolEncoderPrevState;                    // Encoder state (quadrature code)

int nextEncoderState[4] = { 2, 0, 3, 1 };   // quadrature sequence clockwise
int prevEncoderState[4] = { 1, 3, 0, 2 };   // quadrature sequence counter clockwise

const int VolEncoderChA = 2;                // Encoder A-channel arduino pin assignment
const int VolEncoderChB = 4;                // Encoder B-channel arduino pin assignment

// Volume settings:

int VolCurLevel = 0;                        // Current volume setting (0 to 31)
int VolOldLevel = 0;                        // Previous volume setting
int VolCurLevelAcc = 0;                     // Volume level accumulator (used for high resolution encoders)

int mute_state = 0;                         // Mute state (active LOW)
int mute_state_old = 0;

int input = 0;                              // Input index

const int buttonPinInput = 9;               // Input selection push button arduino pin assignment
const int buttonPinMute  = 3;               // Mute push button arduino pin assignment

//Software debounce settings:
unsigned long debounceDelay        = 50;    // debounce time
unsigned long debounceCounterInput = 0;
unsigned long debounceCounterMute  = 0;

const int LEDpin = 8;                       // LED "ON" state indicator pin assignment

//I2C settings for the Attenuator, Indicator and Source Selector:
const int ATT1addr = B00100000;             // Attenuator I2C controller addrees (all swicthes OFF)
const int SELaddr  = B00100111;             // Input selctor I2C controller address (all switches ON)

const int MCP23008_IODIR  = B00000000;      // MCP23008's IODIR internal address
const int MCP23008_GPIO   = B00001001;      // MCP23008's GPIO internal address

bool update_display_input = true;           // LCD display update on input change
bool update_display_attenuator = true;      // LCD display update on volume change

//Active input custom LCD symbols:
byte inputs[4][8] = 
{
{//ONE
  B11011,
  B10011,
  B11011,
  B11011,
  B11011,
  B11011,
  B10001,
  B11111
},
{//TWO
  B10001,
  B01110,
  B11110,
  B11101,
  B11011,
  B10111,
  B00000,
  B11111
},
{//THREE  
  B11111,
  B00000,
  B11101,
  B11011,
  B11101,
  B11110,
  B01110,
  B10001
},
{//FOUR
  B11101,
  B11001,
  B10101,
  B01101,
  B00000,
  B11101,
  B11101,
  B11111
}
};

void setup()
{
  Serial.begin(9600);            // Enable for serial output, e.g. for IR code settings
  myReceiver.enableIRIn();         // Start the receiver
  //Serial.println(F("Ready to receive IR signals")); // Enable for serial output, e.g. for IR code settings

  // LCD display setup:
  lcd.begin (16,2);                // for 16 x 2 LCD module

  for(int n = 0; n < 4; n++)
  {
    lcd.createChar(n, inputs[n]);  // load custom characters to the LCD display memory
  }

  lcd.setBacklightPin(3,POSITIVE); // LCD display backlight enable pin and type of dispay
  lcd.setBacklight(HIGH);          // Enable LCD display backlight
      
  lcd.home ();                     // go the beginning of the first line
  lcd.print("Volume:       dB");   // print volume settings legend 
  lcd.setCursor(0,1);              // go to next line
  lcd.print("Input: ");            // print input selector legend 
  update_display_input = true;    
  update_display_attenuator = true;

  ///////////////////////////////////
  
  // Initializes the wire library
  Wire.begin();
  //Initializes attenuator MCP23008
  Wire.beginTransmission(ATT1addr);       // begin talking to the slave device
  Wire.write(MCP23008_IODIR);             // select the IODIR register
  Wire.write(B00000000);                  // set all port pins to outputs
  Wire.endTransmission();                 // stop talking to device

  Wire.beginTransmission(SELaddr);        // begin talking to the slave device
  Wire.write(MCP23008_IODIR);             // select the IODIR register
  Wire.write(B00000000);                  // set all port pins to outputs
  Wire.endTransmission();                 // stop talking to device

  // Set up for the encoder
  pinMode(VolEncoderChA, INPUT);
  pinMode(VolEncoderChB, INPUT);
  
  digitalWrite(VolEncoderChA, HIGH);      // Enable pull-up resistor
  digitalWrite(VolEncoderChB, HIGH);      // Enable pull-up resistor

  // Turning ON LED
  pinMode(LEDpin, OUTPUT);
  digitalWrite(LEDpin, HIGH);

  // Setting up mute and input selector push buttons
  pinMode(buttonPinMute, INPUT);  
  pinMode(buttonPinInput, INPUT);
  digitalWrite(buttonPinInput, LOW);
  digitalWrite(buttonPinMute, LOW);  
  mute_state = 0;
  SetAttenuatorValue(ATT1addr,VolCurLevel); // Setting initial attenuator value  
  SetInput(SELaddr,input%4);                // Setting default input
}

// Function to increase the volume
void VolumeUp()
{  
    if(mute_state == 0)// if muted then unmute before changing the volume
    {
        mute_state = 1; 
    }
    else if(VolCurLevelAcc < EncoderResolution) // if not at maximum volume continue increasing it (255 corresponds to 64 pulses per revolution encoder)
    {
        VolCurLevelAcc++;
        VolCurLevel = VolCurLevelAcc/EncoderToAttenuationRatio;
    }
}

// Function to decrease the volume
void VolumeDown()
{
    if(VolCurLevelAcc > 0)
    {
        VolCurLevelAcc--;
        VolCurLevel = VolCurLevelAcc/EncoderToAttenuationRatio;
    }
    else // if volume is at minimum then mute
    {
        mute_state = 0;
    }
}

// Enables relay corresponding to selected input
void SetInput(const int address, int input)
{  
    switch(input)
    {
        case 0: 
        {
            Wire.beginTransmission(address);      // start talking to the device
            Wire.write(MCP23008_GPIO);            // select the GPIO register
            Wire.write(B00000001);                // set register value
            Wire.endTransmission();               // stop talking to the device
            break;
        }
        case 1:
        {
            Wire.beginTransmission(address);      // start talking to the device
            Wire.write(MCP23008_GPIO);            // select the GPIO register
            Wire.write(B00000100);                // set register value
            Wire.endTransmission();               // stop talking to the device
            break;
        }
        case 2: 
        {
            Wire.beginTransmission(address);      // start talking to the device
            Wire.write(MCP23008_GPIO);            // select the GPIO register
            Wire.write(B00010000);                // set register value
            Wire.endTransmission();               // stop talking to the device
            break;
        }
        case 3: 
        {
            Wire.beginTransmission(address);      // start talking to the device
            Wire.write(MCP23008_GPIO);            // select the GPIO register
            Wire.write(B01000000);                // set register value
            Wire.endTransmission();               // stop talking to the device
            break;
        }
    }
}
// Sets attenuator relays for selected attenuation
void SetAttenuatorValue(const int address, int attenuation)
{
    // Set mute state
    int loc_attenuation = attenuation;
    bitWrite(loc_attenuation, 5, mute_state);        //set mute state
    Serial.print(loc_attenuation);

    // Set the attenuator to the required value
    Wire.beginTransmission(address);             // start talking to the device
    Wire.write(MCP23008_GPIO);                   // select the GPIO register
    Wire.write(loc_attenuation);                     // set registers
    Wire.endTransmission();                      // stop talking to the device
}

void loop()
{
    mute_state_old = mute_state;
    VolOldLevel = VolCurLevel;

    //IR receiver code. Continue looping until a complete signal received
    if (myReceiver.getResults()) 
    {
      myDecoder.decode();               // Decode it
      //myDecoder.dumpResults(false);   // Print result to serial interface. Uncomment for initial IR remote setup
      switch(myDecoder.value) // Depending on reveived IR code set volume or change input
      {
        // Using NEC codes
        case 0x3434E817: // volume up
        {
          if(mute_state == 0)
          {
            mute_state = 1;
          }
          else if (VolCurLevel < MAXAttenuation)
          {
            VolCurLevel++;
            VolCurLevelAcc += EncoderToAttenuationRatio;
          }
          break;
        }
        case 0x34346897: // volume down
        {
          if(VolCurLevel > 0 and mute_state)
          {
            VolCurLevel--;
            VolCurLevelAcc -= EncoderToAttenuationRatio;
          }
          else if(mute_state)
          {
            mute_state = 0;
          }
          else if(VolCurLevel > 0)
          {
            mute_state = 1;
          }
          break;
        }
        case 0x3434F807: //mute
        {
          mute_state = !mute_state;
          break;       
        }
      }
      myReceiver.enableIRIn();      //Restart IR receiver
    }
  
    // Read rotary encoder and change volume accordingly
    VolEncoderCurrState = (digitalRead(VolEncoderChB) << 1) | digitalRead(VolEncoderChA);
    if (VolEncoderCurrState != VolEncoderPrevState) 
    {
      if(VolEncoderCurrState == nextEncoderState[VolEncoderPrevState]) // Volume Down
      {
        if(VolCurLevelAcc > 0)
        {
          VolCurLevelAcc--;
          VolCurLevel = VolCurLevelAcc/EncoderToAttenuationRatio;
        }
        else // if volume is at minimum then mute
        {
          mute_state = 0;
        }
      } 
      else if (VolEncoderCurrState == prevEncoderState[VolEncoderPrevState]) // Volume Up
      {
        if(mute_state == 0)// if muted then unmute before changing the volume
        {
          mute_state = 1; 
        }
        else if(VolCurLevelAcc < EncoderResolution) // if not at maximum volume continue increasing it (255 corresponds to 64 pulses per revolution encoder)
        {
          VolCurLevelAcc++;
          VolCurLevel = VolCurLevelAcc/EncoderToAttenuationRatio;
        }
      }
      VolEncoderPrevState = VolEncoderCurrState;
  }
     
    // Read the button pin and change input if it is pressed 
    int buttonStateInput = digitalRead(buttonPinInput);

    if(buttonStateInput == HIGH)
    {
      debounceCounterInput++;
    }
    else
    {
      debounceCounterInput = 0;
    }
    
    if(debounceCounterInput == debounceDelay)
    {
      input = (input+1)%4;
      SetInput(SELaddr, input);
      update_display_input = true;
    }
    
    // Read the button pin and change mute state if it is pressed

    int buttonStateMute = digitalRead(buttonPinMute);

    if(buttonStateMute == HIGH)
    {
      debounceCounterMute++;
    }
    else
    {
      debounceCounterMute = 0;
    }
    
    if(debounceCounterMute == debounceDelay)
    {
      mute_state = !mute_state;
    }

    if((VolCurLevel != VolOldLevel) or (mute_state != mute_state_old))
    {
      SetAttenuatorValue(ATT1addr,VolCurLevel); 
      update_display_attenuator = true;       
    }  

    if(update_display_attenuator)  // if attenuation changed, update display
    {
      lcd.setCursor ( 9, 0 );
      if(mute_state == 0)
      {
        lcd.print ("M   ");
      }
      else
      {
        lcd.print ("    ");
      }
  
      if(VolCurLevel == 31)
      {
        lcd.setCursor ( 13, 0 );
      }
      else if(VolCurLevel > 26)
      {
        lcd.setCursor ( 12, 0 );
      }
      else
      {
        lcd.setCursor ( 11, 0 );
      }
      
      lcd.print ((VolCurLevel - MAXAttenuation)*dBperStep);
      update_display_attenuator = false;
    }
    if(update_display_input)
    {
      lcd.setCursor ( 8, 1 );
      lcd.print(" 1 2 3 4 ");
      lcd.setCursor ( 9+input*2, 1 );
      lcd.write (byte (input));
      update_display_input = false;
    }
}
